% 8-queens - what else is there to say
%forward checking
%first fail
%with variable ordering

queens(N,L):-
	length(L,N),
	inlist(L,1..N),
	safe(L),
	reorder(L,L1),
	qlabeling(L1,N).

delay length(nonvar,any).
delay length(any,nonvar).
length([],0).
length([X|Y],N):- less(0,N), plus(N1,1,N), length(Y,N1).

delay less(nonvar,nonvar).
less(X,Y):- X<Y.

inlist([],_).
inlist([X|Y],D):- X in D, inlist(Y,D).

labeling([]).
labeling([F|T]):- deleteff(X,[F|T],Ls), indomain(X), labeling(Ls).

indomain(X):- domain(X,L), member(X,L).

member(X,[X|_]).
member(X,[_|Y]):- member(X,Y).

%delay safe(nonvar).
safe([]).
safe([X|Xs]):-
	noattack(X,Xs),
	safe(Xs).

noattack(X,L):- noattack(X,L,1).

%delay noattack(any,nonvar,any).
noattack(X,[],Nb).
noattack(X,[Y|Ys],Nb):-
	notplus(X, 0, Y),
	notplus(X,Nb,Y),
	notplus(Y,Nb,X),
	Nb1 is Nb+1,
	noattack(X,Ys,Nb1).

%reorder(L1,L2) orders the middle elements of L1 as the first of L2
%e.g. reorder([1,2,3,4,5],[3,2,4,1,5])
reorder(L1,L2):-
	length(L1,N),
	N2 is N/2,
	length(F,N2),
	append(F, R, L1),
	rev(F,RF),
	interleave(R,RF,L2).

rev(L1,L2):- rev(L1,[],L2).

rev([],L,L).
rev([X|L],L2,L3):- rev(L,[X|L2],L3).

interleave([],[],[]).
interleave([],[H|T],[H|T]).
interleave([H|T],[],[H|T]).
interleave([X|T1],[Y|T2],[X,Y|T3]):- interleave(T1,T2,T3).

%not needed
delete_middle(X,L,R):- 
	length(L,N),
	N2 is N/2,
	length(F,N2),
	append(F,[X|T],L),
	append(F,T,R).

append([],L,L).
append([X|L1],L2,[X|L3]):- append(L1,L2,L3).

integers(0,[]).
integers(N,[N|T]):- less(0,N), plus(N1,1,N), integers(N1,T).

qlabeling(L,N):- integers(N,L1), reorder(L1,L2), labeling(L,L2).

labeling([],_).
labeling([F|T],L):- deleteff(X,[F|T],R), member(X,L), labeling(R,L).

%% delay noteq(nonvar,any).
%% delay noteq(any,nonvar).
%% noteq(X,Y):- notplus(X,0,Y).
%% 
%% delay notsum(any,nonvar,nonvar).
%% delay notsum(nonvar,any,nonvar).
%% delay notsum(nonvar,nonvar,any).
%% notsum(X,Y,Z):- notplus(X,Y,Z).

allq(N) :- queens(N, L), fail.
allq(N).

top:-
	read(N, []),
	cputime(T0),
	queens(N,L),
	cputime(T1),
	Time is 20 * (T1-T0),
	write((time=Time, queens(N,L))),nl.
