% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 8-queens - what else is there to say
% ***** forward checking
% ***** first fail principle
% ***** with variable ordering
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

top :-
	x_open,
	write('NUMBER OF QUEENS: '), read(N, []),
	draw_board(N, P, W),
%	draw_board(N, P, W),
	write_title(N),
	queens(N, L, P, W),
	show_solved,
	x_close.

top :-
	write('NUMBER OF QUEENS: '), read(N, []),
	cputime(T1),
	queens(N, L),
	cputime(T2),
	plus(T1,T,T2),
	write(time=T),nl,
	write(L),nl.

x_open :-
	x_init_win('Aristo Version 1.0', 0, 0, 1000, 1000),
	x_clear_win.

x_close :-
	x_close_win.

write_title(N) :-
	x_set_font('*times*bold-r*-34-*'),
	x_draw_string('N-Queens-Problem', 340, 70).

show_solved :-
	x_draw_string('solved !', 450, 970),
	get_character.

get_character :-
	write('--> '), read(_, []).

% draw_board(+Queens, -UpperCorner, -FieldWidth).
draw_board(N, P, W) :-
	W is 800 / N,
	A is N * W,
	B is 1000 - A,
	P is B / 2,
%%%	draw_all_rectangles(N, P, W).
	draw_lines(N, P, P, A, W).

draw_lines(0, X, Y, L, _) :-
	x_draw_line(X, Y, 0, L),
	x_draw_line(Y, X, L, 0).

draw_lines(N, X, Y, L, W) :-
	N > 0,
	x_draw_line(X, Y, 0, L),
	x_draw_line(Y, X, L, 0),
	X1 is X + W,
	N1 is N - 1,
	draw_lines(N1, X1, Y, L, W).


draw_queens(L, P, W) :-
	draw_queens(L, P, P, W).

draw_queens([], _, _, _).
draw_queens([F|R], X, Y, W) :-
	draw_queen(F, X, Y, W),
	X1 is X + W,
	draw_queens(R, X1, Y, W).

delay draw_queen(ground, any, any, any).
draw_queen(V, X, Y0, W) :-
	Y is (((V - 1) * W) + Y0),
	x_fill_rectangle(X, Y, W, W).
draw_queen(V, X, Y0, W) :-
	Y is (((V - 1) * W) + Y0),
	x_clear_rectangle(X, Y, W, W),
	fail.

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% n-queens specific code
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

queens(N, L, P, W):-
	length(L, N),
	L in 1 .. N,
	draw_queens(L, P, W),	%%% graphic support
	safe(L),
	reorder(L,L1),
	qlabelingx(L1, N).

queens(N, L):-
	length(L, N),
	L in 1 .. N,
	safe(L),
	reorder(L,L1),
	qlabelingx(L1, N).

%delay length(nonvar,any).
%delay length(any,nonvar).
%length([],0).
%length([X|Y],N):- less(0,N), plus(N1,1,N), length(Y,N1).

delay less(nonvar,nonvar).
less(X,Y):- X<Y.

labelingx([]).
labelingx([F|T]):- deleteff(X,[F|T],Ls,[size,change]), indomain(X), labelingx(Ls).

%indomain(X):- domain(X,L), member(X,L).

%member(X,[X|_]).
%member(X,[_|Y]):- member(X,Y).

qlabelingx(L,N):- integers(N,L1), reorder(L1,L2), labelingx(L,L2).

labelingx([],_).
labelingx([F|T],L):- 
	deleteff(X,[F|T],R, [size]), 
	member(X,L), 
	labelingx(R,L).

rlabelingx(L) :- varff(X, L), !, rindomain(X), rlabelingx(L).
rlabelingx(_).

rindomain(X):- domain(X,L), once(reorder(L, L1)), member(X,L1).

%once(X) :- call(X), !.
%once(X) :- write('should not come to '), write(X), nl.

%alldifferent(L) :-
%	createalldiff(L, P),
%	checkalldiffs(L, 0, P).

%checkalldiffs([], _, _).
%checkalldiffs([H|T], N, P) :-
%	checkalldiff(H, N, P),
%	N1 is N + 1,
%	checkalldiffs(T, N1, P).

safe(L):-
	alldifferent(L),
	nodiags(L,L,1).

nodiags([],_,_).
nodiags([C|T],L,R):- nodiag(C,R,L), plus(R,1,R1), nodiags(T,L,R1).

delay nodiag(nonvar,any,any).
nodiag(C,R,L):- nodiag(C,R,L,1).

nodiag(_,_,[],_).
nodiag(C,R,[H|T],R):- !,plus(R,1,R1), nodiag(C,R,T,R1).
nodiag(C,R,[H|T],R1):-
	notplus(R,0,R1),
	plus(R,Dif,R1),
	notplus(C,Dif,H),
	notplus(H,Dif,C),
	plus(R1,1,R2), nodiag(C,R,T,R2).

safe([]).
safe([X|Xs]):-
	noattack(X,Xs),
	safe(Xs).

noattack(X,L):- noattack(X,L,1).

noattack(X,[],Nb).
noattack(X,[Y|Ys],Nb):-
	notplus(X,Nb,Y),
	notplus(Y,Nb,X),
	Nb1 is Nb+1,
	noattack(X,Ys,Nb1).

%reorder(L1,L2) orders the middle elements of L1 as the first of L2
%e.g. reorder([1,2,3,4,5],[3,2,4,1,5])
reorder(L1,L2):-
	length(L1,N),
	N2 is N/2,
	length(F,N2),
	append(F, R, L1),
	rev(F,RF),
	interleave(R,RF,L2).

%rev(L1,L2):- rev(L1,[],L2).

%rev([],L,L).
%rev([X|L],L2,L3):- rev(L,[X|L2],L3).

interleave([],[],[]).
interleave([],[H|T],[H|T]).
interleave([H|T],[],[H|T]).
interleave([X|T1],[Y|T2],[X,Y|T3]):- interleave(T1,T2,T3).

%not needed
delete_middle(X,L,R):- 
	length(L,N),
	N2 is N/2,
	length(F,N2),
	append(F,[X|T],L),
	append(F,T,R).

%append([],L,L).
%append([X|L1],L2,[X|L3]):- append(L1,L2,L3).

integers(0,[]).
integers(N,[N|T]):- less(0,N), plus(N1,1,N), integers(N1,T).



