constraints(
	[S0_1, S0_2, S1_1, S1_2,
	 T0_A, T0_B, T0_C, T0_D,
	 T1_A, T1_B, T1_C, T1_D,
	 T2_A, T2_B, T2_D], 
	MsgLst,
	End
) :-


	task(S0_1, TS0_1, 0, 2),
	task(S0_2, TS0_2, 0, 2),
	task(S1_1, TS1_1, 1, 2),
	task(S1_2, TS1_2, 1, 2),

	task(T0_A, TT0_A, 0, 2),
	task(T0_B, TT0_B, 1, 2),
	task(T0_C, TT0_C, 0, 2),
	task(T0_D, TT0_D, 0, 2),

	task(T1_A, TT1_A, 1, 2),
	task(T1_B, TT1_B, 0, 2),
	task(T1_C, TT1_C, 1, 2),
	task(T1_D, TT1_D, 1, 2),

	task(T2_A, TT2_A, 2, 2),
	task(T2_B, TT2_B, 2, 2),
	task(T2_D, TT2_D, 2, 2),

	%%% tasks on same component must not run at the same time
	disjoint([S0_1, S0_2, S1_1, S1_2, T0_A, T0_B, T0_C, T0_D,
		   T1_A, T1_B, T1_C, T1_D, T2_A, T2_B, T2_D]),

	message(T0_A, T0_B, [], Msg1),
	message(T0_A, T0_C, Msg1, Msg2),
	message(T0_B, T0_D, Msg2, Msg3),
	message(T0_C, T0_D, Msg3, Msg4),
	message(T1_A, T1_B, Msg4, Msg5),
	message(T1_A, T1_C, Msg5, Msg6),
	message(T1_C, T1_D, Msg6, Msg7),
	message(T1_B, T1_D, Msg7, Msg8),
	message(T2_A, T2_B, Msg8, Msg9),
	message(T2_A, T2_D, Msg9, MsgLst),

	alldifferent(MsgLst),

	TT0_D + 2 #=< TT0_A + 14,
	TT1_D + 2 #=< TT1_A + 14,
	TT2_D + 2 #=< TT2_A + 32,
	
	TS0_1 #>= 0, TS0_1 #< 8,
	TS0_2 #= TS0_1 + 8,
	TS1_2 #= TS1_1 + 8,
	TS1_1 #>= 0, TS1_1 #< 8,

	[End] in 0 .. 15,
	TS0_2 + 2 #=< End,
	TS1_2 + 2 #=< End,
	TT0_D + 2 #=< End,
	TT1_D + 2 #=< End,
	TT2_D + 2 #=< End.

task(task(Start, Comp, Duration), Start, Comp, Duration) :-
	[Start] in 0 .. 15,
	Start + Duration #=< 15.

message(task(T1, M, D1), task(T2, M, D2), MsgLst, MsgLst) :-
	!,
	T2 #>= T1 + D1.

message(task(T1, M1, D1), task(T2, M2, D2), MsgLst, [Msg|MsgLst]) :-
	M1 \= M2,
	T2 #> T1 + D1,
	[Msg] in 0 .. 15,
	Msg #>= T1 + D1,
	Msg + 1 #=< T2.

disjoint([]).
disjoint([H|T]) :-
	disjoint(T, H),
	disjoint(T).

disjoint([], _).
disjoint([F|R], X) :-
	same_comp(F, X),
	disjoint(R, X).

same_comp(task(T1, M, D1), task(T2, M, D2)) :-
	!,
	disjoint(T1, D1, T2, D2).
same_comp(task(T1, M1, D1), task(T2, M2, D2)) :-
	M1 \= M2.


top(N, [Tasks, Messages, End]) :-
	constraints(Tasks, Messages, End),
	taskvars(Tasks, TaskVars),
	append(TaskVars, Messages, Vars),
	[End] in 0 .. N,
	labeling(Vars, [size, constraints]),
	$printdelays, !.
%	writelist(TaskVars),
%	writelist(Messages).

taskvars([], []).
taskvars([task(V, _, _)|RT], [V|RV]) :-
	taskvars(RT, RV).

writelist([]) :-
	write('-->'), read(_, []).

writelist([F|R]) :-
	domain(F, D),
	write(F), write(' in '), write(D), nl,
	writelist(R).

